test("distance()", function() {
  equal(distance(0, 0, 0, 0), 0);
  equal(distance(0, 0, 1, 0), 1);
  equal(distance(0, 0, -1, 0), 1);
  equal(distance(0, 0, 0, 1), 1);
  equal(distance(0, 0, 0, -1), 1);
  equal(distance(1, 0, 0, 0), 1);
  equal(distance(-1, 0, 0, 0), 1);
  equal(distance(0, 1, 0, 0), 1);
  equal(distance(0, -1, 0, 0), 1);
  equal(distance(0, 0, 4, 3), 5);
});

test("projection()", function() {
  deepEqual(projection(1, 1, 1, 0), { x:1, y:0 });
  deepEqual(projection(1, 1, -1, 0), { x:1, y:0 });
  deepEqual(projection(1, 1, 0, 1), { x:0, y:1 });
  deepEqual(projection(1, 1, 0, -1), { x:0, y:1 });
  deepEqual(projection(1, 1, 1, 1), { x:1, y:1 });
  deepEqual(projection(1, 1, 0, 0), { x:NaN, y:NaN });
});

test("limit_value()", function() {
  equal(limit_value(-1, 0, 10), 0);
  equal(limit_value(0, 0, 10), 0);
  equal(limit_value(1, 0, 10), 1);
  equal(limit_value(9, 0, 10), 9);
  equal(limit_value(10, 0, 10), 10);
  equal(limit_value(11, 0, 10), 10);
});

var timer_id;
asyncTest('ready()', 1, function() {
  timer_id = window.setInterval(function() {
    if ($ball_img != null) {
      ok(true, 'ready()');
      window.clearInterval(timer_id);
      start();
    }
  }, 100);
});

function change_hole_n(n, exp) {
  $('#hole_n').spinner('value', n);
  on_reset();
  window.setTimeout(function() {
    equal($('#hole_n').spinner('value'), exp);
    start();
  }, 200);
};

asyncTest("change holes", 1, function() {
  change_hole_n(HOLE_MIN - 1, HOLE_MIN);
});
asyncTest("change holes", 1, function() {
  change_hole_n(HOLE_MIN, HOLE_MIN);
});
asyncTest("change holes", 1, function() {
  change_hole_n(HOLE_MIN + 1, HOLE_MIN + 1);
});
asyncTest("change holes", 1, function() {
  change_hole_n(HOLE_MAX - 1, HOLE_MAX - 1);
});
asyncTest("change holes", 1, function() {
  change_hole_n(HOLE_MAX, HOLE_MAX);
});
asyncTest("change holes", 1, function() {
  change_hole_n(HOLE_MAX + 1, HOLE_MAX);
});

function change_goal_n(n, exp) {
  $('#goal_n').spinner('value', n);
  on_reset();
  window.setTimeout(function() {
    equal($('#goal_n').spinner('value'), exp);
    start();
  }, 200);
};

asyncTest("change goals", 1, function() {
  change_goal_n(GOAL_MIN - 1, GOAL_MIN);
});
asyncTest("change goals", 1, function() {
  change_goal_n(GOAL_MIN, GOAL_MIN);
});
asyncTest("change goals", 1, function() {
  change_goal_n(GOAL_MIN + 1, GOAL_MIN + 1);
});
asyncTest("change goals", 1, function() {
  change_goal_n(GOAL_MAX - 1, GOAL_MAX - 1);
});
asyncTest("change goals", 1, function() {
  change_goal_n(GOAL_MAX, GOAL_MAX);
});
asyncTest("change goals", 1, function() {
  change_goal_n(GOAL_MAX + 1, GOAL_MAX);
});

asyncTest("change holes=5", 1, function() {
  change_hole_n(7, 7);
});

test("click outside", function() {
    var e = { preventDefault: function() {},
              stopPropagation: function() {},
              pageX: 0,
              pageY: 0 };
    equal(dragging, false);
    e.pageX = STAGE_W;
    e.pageY = 0;
    on_mousedown(e);
    equal(dragging, false);
    e.pageX = 0;
    e.pageY = STAGE_H;
    on_mousedown(e);
    equal(dragging, false);
});

test("mouse event disorder", function() {
    var e = { preventDefault: function() {},
              stopPropagation: function() {},
              pageX: 0,
              pageY: 0 };
    equal(dragging, false);
    e.pageX = STAGE_W;
    e.pageY = 0;
    on_mousemove(e);
    equal(dragging, false);
    on_mouseup(e);
    equal(dragging, false);
});

var launch_count;
asyncTest("fast launch", 20, function() {
  launch_count = 0;
  timer_id = window.setInterval(function() {
      var e = { preventDefault: function() {},
                stopPropagation: function() {},
                pageX: 0,
                pageY: STAGE_H - 10 };
      on_mousedown(e);
      on_mousemove(e);
      on_mouseup(e);
      ok(true, 'fast launch');
      if (++launch_count >= 20) {
          window.clearInterval(timer_id);
          start();
      }
  }, 50)
});

asyncTest("launch", 720, function() {
  launch_count = 0;
  timer_id = window.setInterval(function() {
      if (balls.length < BALL_N) {
          var e = { preventDefault: function() {},
                    stopPropagation: function() {},
                    pageX: STAGE_W / 2,
                    pageY: STAGE_H - 10 };
          on_mousedown(e);
          e.pageX = 4 * (launch_count % 120);
          e.pageY = STAGE_H - 40 * ((launch_count / 120) >> 0)
          on_mousemove(e);
          on_mouseup(e);
          ok(true, 'launch');
          if (++launch_count >= 720) {
              window.clearInterval(timer_id);
              start();
          }
      }
  }, 50)
});

test("reset()", function() {
    on_reset();
    equal(total, 0);
    equal(point, 0);
});

